@@ -6,13 +6,13 @@ use Sub::Defer;
 use Moo::_Utils qw(_getstash _getglob);
 use Moo::_mro;
 use Scalar::Util qw(weaken);
+use Carp qw(croak);
+BEGIN { our @CARP_NOT = qw(Sub::Defer) }
 BEGIN {
   local $Moo::sification::disabled = 1;
   require Moo;
   Moo->import;
 }
-use Carp qw(croak);
-BEGIN { our @CARP_NOT = qw(Sub::Defer) }
 
 sub register_attribute_specs {
   my ($self, @new_specs) = @_;
@@ -86,6 +86,7 @@ sub install_delayed {
   my $package = $self->{package};
   my (undef, @isa) = @{mro::get_linear_isa($package)};
   my $isa = join ',', @isa;
+  my (undef, $from_file, $from_line) = caller(Carp::short_error_loc());
   my $constructor = defer_sub "${package}::new" => sub {
     my (undef, @new_isa) = @{mro::get_linear_isa($package)};
     if (join(',', @new_isa) ne $isa) {
@@ -96,7 +97,7 @@ sub install_delayed {
         $expected_new ||= 'none';
         croak "Expected parent constructor of $package to be"
         . " $expected_new, but found $found_new: changing the inheritance"
-        . " chain (\@ISA) at runtime is unsupported";
+        . " chain (\@ISA) at runtime (after $from_file line $from_line) is unsupported";
       }
     }
 
@@ -144,7 +145,8 @@ sub generate_method {
   my $into_buildargs = $into->can('BUILDARGS');
 
   my $body
-    = '    my $class = ref($_[0]) ? ref(shift) : shift;'."\n"
+    = '    my $invoker = shift;'."\n"
+    . '    my $class = ref($invoker) ? ref($invoker) : $invoker;'."\n"
     . $self->_handle_subconstructor($into, $name)
     . ( $into_buildargs && $into_buildargs != \&Moo::Object::BUILDARGS
       ? $self->_generate_args_via_buildargs
