@@ -5,26 +5,34 @@ no warnings 'once'; # guard against -w
 sub _getglob { \*{$_[0]} }
 sub _getstash { \%{"$_[0]::"} }
 
-use constant lt_5_8_3 => ( $] < 5.008003 or $ENV{MOO_TEST_PRE_583} ) ? 1 : 0;
-use constant can_haz_subname => eval { require Sub::Name };
+use Moo::_strictures;
+
+BEGIN {
+  my ($su, $sn);
+  $su = $INC{'Sub/Util.pm'} && defined &Sub::Util::set_subname
+    or $sn = $INC{'Sub/Name.pm'}
+    or $su = eval { require Sub::Util; } && defined &Sub::Util::set_subname
+    or $sn = eval { require Sub::Name; };
+
+  *_subname = $su ? \&Sub::Util::set_subname
+            : $sn ? \&Sub::Name::subname
+            : sub { $_[1] };
+  *_CAN_SUBNAME = ($su || $sn) ? sub(){1} : sub(){0};
+}
 
-use strictures 1;
 use Module::Runtime qw(use_package_optimistically module_notional_filename);
 
 use Devel::GlobalDestruction ();
 use Exporter qw(import);
-use Moo::_mro;
 use Config;
+use Carp qw(croak);
 
 our @EXPORT = qw(
     _getglob _install_modifier _load_module _maybe_load_module
-    _get_linear_isa _getstash _install_coderef _name_coderef
-    _unimport_coderefs _in_global_destruction _set_loaded
+    _getstash _install_coderef _name_coderef
+    _unimport_coderefs _set_loaded
 );
 
-sub _in_global_destruction ();
-*_in_global_destruction = \&Devel::GlobalDestruction::in_global_destruction;
-
 sub _install_modifier {
   my ($into, $type, $name, $code) = @_;
 
@@ -33,14 +41,13 @@ sub _install_modifier {
     Sub::Defer::undefer_sub($to_modify);
   }
 
+  require Class::Method::Modifiers;
   Class::Method::Modifiers::install_modifier(@_);
 }
 
-our %MAYBE_LOADED;
-
 sub _load_module {
   my $module = $_[0];
-  my $file = module_notional_filename($module);
+  my $file = eval { module_notional_filename($module) } or croak $@;
   use_package_optimistically($module);
   return 1
     if $INC{$file};
@@ -53,9 +60,10 @@ sub _load_module {
   return 1
     if $INC{"Moose.pm"} && Class::MOP::class_of($module)
     or Mouse::Util->can('find_meta') && Mouse::Util::find_meta($module);
-  die $error;
+  croak $error;
 }
 
+our %MAYBE_LOADED;
 sub _maybe_load_module {
   my $module = $_[0];
   return $MAYBE_LOADED{$module}
@@ -73,10 +81,6 @@ sub _set_loaded {
   $INC{Module::Runtime::module_notional_filename($_[0])} ||= $_[1];
 }
 
-sub _get_linear_isa {
-  return mro::get_linear_isa($_[0]);
-}
-
 sub _install_coderef {
   my ($glob, $code) = (_getglob($_[0]), _name_coderef(@_));
   no warnings 'redefine';
@@ -93,7 +97,7 @@ sub _install_coderef {
 
 sub _name_coderef {
   shift if @_ > 2; # three args is (target, name, sub)
-  can_haz_subname ? Sub::Name::subname(@_) : $_[1];
+  _CAN_SUBNAME ? _subname(@_) : $_[1];
 }
 
 sub _unimport_coderefs {
